// helpers.go
package expr

import (
	"fmt"
	"strings"
)

func EvalString(ctx ExprContext, source string) (result any, err error) {
	var tree *ast

	r := strings.NewReader(source)
	scanner := NewScanner(r, DefaultTranslations())
	parser := NewParser(ctx)

	if tree, err = parser.Parse(scanner); err == nil {
		result, err = tree.eval(ctx, true)
	}
	return
}

type Arg struct {
	Name  string
	Value any
}

func EvalStringA(source string, args ...Arg) (result any, err error) {
	return EvalStringV(source, args)
}

func EvalStringV(source string, args []Arg) (result any, err error) {
	ctx := NewSimpleFuncStore()
	for _, arg := range args {
		if isFunc(arg.Value) {
			if f, ok := arg.Value.(FuncTemplate); ok {
				functor := &simpleFunctor{f: f}
				ctx.RegisterFunc(arg.Name, functor, 0, -1)
			} else {
				err = fmt.Errorf("invalid function specification: %q", arg.Name)
			}
		} else if integer, ok := anyInteger(arg.Value); ok {
			ctx.SetVar(arg.Name, integer)
		} else if float, ok := anyFloat(arg.Value); ok {
			ctx.SetVar(arg.Name, float)
		} else if _, ok := arg.Value.(string); ok {
			ctx.SetVar(arg.Name, arg.Value)
		} else if _, ok := arg.Value.(bool); ok {
			ctx.SetVar(arg.Name, arg.Value)
		} else {
			err = fmt.Errorf("unsupported type %T specified for item %q", arg.Value, arg.Name)
		}
	}

	if err == nil {
		result, err = EvalString(ctx, source)
	}
	return
}
